__author__ = "Ben Woodcroft"
__copyright__ = "Copyright 2014-2015"
__credits__ = ["Ben Woodcroft"]
__license__ = "LGPL3"
__maintainer__ = "Ben Woodcroft"
__email__ = "b.woodcroft near uq.net.au"
__status__ = "Development"

from .version import __version__
